"""
Contains different data classes that more easy working that regular dicts.
"""

from .age_raiting import *
from .description import *
from .id import *
from .name import *
from .poster import *
from .production_status import *
from .raiting import *
from .review import *
from .types import *
from .url import *
from .movie import *